include Stdlib.Result

module O = struct
  let ( >>= ) = Stdlib.Result.bind
  let ( >>| ) res f = Stdlib.Result.map f res
  let ( let* ) = ( >>= )
  let ( let+ ) = ( >>| )
end

module List = struct
  open O
  module List = Stdlib.List

  let map ~f l =
    let rec aux acc = function
      | [] -> Ok (List.rev acc)
      | hd :: tl -> (
          match f hd with
          | Ok hd' -> aux (hd' :: acc) tl
          | Error err -> Error err)
    in
    aux [] l

  let rec iter ~f l =
    match l with
    | [] -> Ok ()
    | hd :: tl ->
        let* () = f hd in
        iter ~f tl

  let rec fold_left t ~f ~init =
    match t with
    | [] -> Ok init
    | x :: xs ->
        let* init = f init x in
        fold_left xs ~f ~init

  let rec exists t ~f =
    match t with
    | [] -> Ok false
    | x :: xs ->
        let* p = f x in
        if p then Ok true else exists xs ~f
end
